pub struct Tx<DB: Database, E = Error>(/* private fields */);Expand description
An actix extractor for a database transaction.
&mut Tx implements sqlx::Executor so it can be used directly with sqlx::query()
(and sqlx::query_as(), the corresponding macros, etc.):
use actix_sqlx_tx::Tx;
use sqlx::Sqlite;
async fn handler(mut tx: Tx<Sqlite>) -> Result<(), sqlx::Error> {
sqlx::query("...").execute(&mut tx).await?;
/* ... */
}It also implements Deref<Target = sqlx::Transaction> and DerefMut, so you can call
methods from Transaction and its traits:
use actix_sqlx_tx::Tx;
use sqlx::{Acquire as _, Sqlite};
async fn handler(mut tx: Tx<Sqlite>) -> Result<(), sqlx::Error> {
let inner = tx.begin().await?;
/* ... */
}The E generic parameter controls the error type returned when the extractor fails. This can be
used to configure the error response returned when the extractor fails:
use actix_web::{
http::{header::ContentType, StatusCode},
HttpResponse, ResponseError,
};
use actix_sqlx_tx::Tx;
use sqlx::Sqlite;
[derive(Debug)]
struct MyError(actix_sqlx_tx::Error);
// The error type must implement From<actix_sqlx_tx::Error>
impl From<actix_sqlx_tx::Error> for MyError {
fn from(error: actix_sqlx_tx::Error) -> Self {
Self(error)
}
}
// The error type must implement ResponseError
impl ResponseError for MyError {
fn error_response(&self) -> HttpResponse {
HttpResponse::build(self.status_code())
.insert_header(ContentType::html())
.body(self.to_string())
}
fn status_code(&self) -> StatusCode {
StatusCode::INTERNAL_SERVER_ERROR
}
}
async fn handler(tx: Tx<Sqlite, MyError>) {
/* ... */
}Implementations
Sourceimpl<DB: Database, E> Tx<DB, E>
impl<DB: Database, E> Tx<DB, E>
Sourcepub async fn commit(self) -> Result<(), Error>
pub async fn commit(self) -> Result<(), Error>
Explicitly commit the transaction.
By default, the transaction will be committed when a successful response is returned
(specifically, when the Service middleware intercepts an HTTP 2XX
response). This method allows the transaction to be committed explicitly.
Note: trying to use the Tx extractor again after calling commit will currently
generate Error::OverlappingExtractors errors. This may change in future.